'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _d3Force = require('d3-force');

var _d3Zoom = require('d3-zoom');

var d3Zoom = _interopRequireWildcard(_d3Zoom);

var _d3Drag = require('d3-drag');

var d3Drag = _interopRequireWildcard(_d3Drag);

var _d3Selection = require('d3-selection');

var _default = require('./default');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Simulation = function () {
  function Simulation(options, simulation) {
    _classCallCheck(this, Simulation);

    _initialiseProps.call(this);

    var _simulation = simulation || (0, _d3Force.forceSimulation)();
    this.simulation = _simulation;
    this.setSimulationLayout(options);
  }

  _createClass(Simulation, [{
    key: 'setSimulationLayout',
    value: function setSimulationLayout(_ref) {
      var velocityDecay = _ref.velocityDecay,
          nodeIdKey = _ref.nodeIdKey,
          linkDistance = _ref.linkDistance,
          collideRadius = _ref.collideRadius,
          collideStrength = _ref.collideStrength,
          chargeStrength = _ref.chargeStrength,
          alphaDecay = _ref.alphaDecay,
          alphaMin = _ref.alphaMin,
          XYCenter = _ref.XYCenter,
          width = _ref.width,
          height = _ref.height,
          sourceKey = _ref.sourceKey,
          targetKey = _ref.targetKey;

      this.width = width;
      this.height = height;
      this.nodeIdKey = nodeIdKey;
      if (sourceKey) {
        this.sourceKey = sourceKey;
      }
      if (targetKey) {
        this.targetKey = targetKey;
      }
      if (velocityDecay) {
        this.simulation.velocityDecay(velocityDecay);
      }
      if (alphaMin) {
        this.simulation.alphaMin(alphaMin);
      }
      if (alphaDecay) {
        this.simulation.alphaDecay(alphaDecay);
      }
      var _forceLink = (0, _d3Force.forceLink)();
      if (nodeIdKey) {
        _forceLink.id(function (d) {
          return d[nodeIdKey];
        });
      }
      if (linkDistance) {
        _forceLink.distance(linkDistance);
      }
      this.simulation.force('link', _forceLink);
      // this.simulation.force('center', forceCenter());

      if (collideRadius || collideStrength) {
        var _forceCollide = (0, _d3Force.forceCollide)();
        if (collideRadius) {
          _forceCollide.radius(typeof collideRadius === 'function' ? function (d) {
            return collideRadius(d);
          } : collideRadius);
        }
        if (collideStrength) {
          _forceCollide.strength(collideStrength);
        }
        this.simulation.force('collide', _forceCollide);
      } else {
        this.simulation.force('collide', null);
      }

      if (chargeStrength) {
        this.simulation.force("charge", (0, _d3Force.forceManyBody)().strength(chargeStrength));
      }
      if (XYCenter) {
        this.simulation.force('x', (0, _d3Force.forceX)(XYCenter && XYCenter.x || undefined));
        this.simulation.force('y', (0, _d3Force.forceY)(XYCenter && XYCenter.y || undefined));
      } else {
        this.simulation.force('x', null);
        this.simulation.force('y', null);
      }
    }
  }, {
    key: 'initNodes',
    value: function initNodes(nodes) {
      var nodeIdKey = this.nodeIdKey,
          width = this.width,
          height = this.height;

      var nodesMap = {};
      var originNodes = [];
      nodes.forEach(function (node) {
        node.x = node.x || width / 2;
        node.y = node.y || height / 2;
        nodesMap[node[nodeIdKey]] = node;
        originNodes.push(node);
      });
      this.nodesMap = nodesMap;
      this.nodes = originNodes;
    }
  }, {
    key: 'initLinks',
    value: function initLinks(links) {
      var _this = this;

      var nodeIdKey = this.nodeIdKey,
          sourceKey = this.sourceKey,
          targetKey = this.targetKey;

      var newLinks = [];
      links.forEach(function (link) {
        var source = link[sourceKey],
            target = link[targetKey];
        if ((typeof source === 'undefined' ? 'undefined' : _typeof(source)) === 'object') {
          source = source[nodeIdKey];
        }
        if ((typeof target === 'undefined' ? 'undefined' : _typeof(target)) === 'object') {
          target = target[nodeIdKey];
        }
        var sourcePush = _this.nodesMap[source];
        var targetPush = _this.nodesMap[target];
        if (sourcePush && targetPush) {
          link.source = sourcePush;
          link.target = targetPush;
          newLinks.push(link);
        }
      });
      this.links = newLinks;
    }
  }, {
    key: 'setNodesLinks',
    value: function setNodesLinks(nodes, links, alpha) {
      this.initNodes(nodes);
      this.initLinks(links);
      this.simulation.nodes(this.nodes).force('link').links(this.links);
      if (alpha) {
        this.start(alpha);
      }
    }
  }, {
    key: 'start',
    value: function start(alpha) {
      var _alpha = this.simulation.alpha() + alpha;
      this.simulation.alpha(_alpha > 1 ? 1 : alpha).restart();
    }
  }, {
    key: 'initDrag',
    value: function initDrag() {
      var _this2 = this;

      var event = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

      var drag = d3Drag.drag().on('start', function (d) {
        if (event.isDrag && event.isDrag(d) || !event.isDrag) {
          if (!_d3Selection.event.active) {
            _this2.simulation.alphaTarget(0.5).restart();
          }
          event.start && event.start(d);
        }
      }).on('drag', function (d) {
        if (event.isDrag && event.isDrag(d) || !event.isDrag) {
          d.fx = d.x = _d3Selection.event.x;
          d.fy = d.y = _d3Selection.event.y;
          event.drag && event.drag(d);
        }
      }).on('end', function (d) {
        if (event.isDrag && event.isDrag(d) || !event.isDrag) {
          if (!_d3Selection.event.active) {
            _this2.simulation.alphaTarget(0);
          }
          d.fx = null;
          d.fy = null;
          event.end && event.end(d);
        }
      });
      this.drag = drag;
    }
  }, {
    key: 'setTransform',
    value: function setTransform(transform, scale) {
      this.translate = transform;
      this.scale = scale;
    }
  }, {
    key: 'initZoom',
    value: function initZoom(event, scaleExtent) {
      var _this3 = this;

      var isZoom = event.isZoom || _default.noop;
      var zoom = d3Zoom.zoom().on('start', function (d) {
        event.start && event.start(d);
      }).on('zoom', function () {
        if (isZoom(_d3Selection.event) !== false) {
          var transform = _d3Selection.event.transform;
          var translate = [transform.x, transform.y],
              scale = transform.k;
          _this3.setTransform(translate, scale);
          event.zoom && event.zoom({ translate: translate, scale: scale });
        }
      }).on('end', function (d) {
        event.end && event.end(d);
      });
      if (scaleExtent) {
        zoom.scaleExtent(scaleExtent);
      }
      this.zoom = zoom;
    }
  }]);

  return Simulation;
}();

var _initialiseProps = function _initialiseProps() {
  var _this4 = this;

  this.sourceKey = 'source';
  this.targetKey = 'target';
  this.width = _default.WIDTH;
  this.height = _default.HEIGHT;
  this.nodeIdKey = _default.NODE_ID_KEY;
  this.nodes = [];
  this.links = [];
  this.nodesMap = {};
  this.translate = [0, 0];
  this.scale = 1;

  this.tick = function (event) {
    _this4.simulation.on('tick', function () {
      event.tick && event.tick(_this4.simulation.alpha());
    }).on('end', function () {
      event.end && event.end(_this4.simulation.alpha());
    });
  };

  this.execute = function () {
    var simulation = _this4.simulation,
        nodes = _this4.nodes;

    simulation.stop();
    for (var i = 0, n = Math.ceil(Math.log(simulation.alphaMin()) / Math.log(1 - simulation.alphaDecay())); i < n; ++i) {
      simulation.tick();
    }
  };
};

exports.default = Simulation;
//# sourceMappingURL=simulation.js.map